indexing
   description: "gtk clist"
   status: "You may be able to redistribute and/or modify this program.% 
          % See the README for details."
   copyright: "Copyright (C) 1999 Paul C. Janzen"
   author: "Paul C. Janzen"
   id: "$Id: gtk_clist.e,v 1.11 1999/01/18 19:54:56 pcj Exp $"

class GTK_CLIST
inherit 
   GTK_CLIST_CTREE_TYPE
creation
   make, 
   make_with_titles, 
   create

feature -- Creation

   make (columns : INTEGER) is
      do
         gtkobject := ext_clist_new ( columns )
      end

   make_with_titles (columns : INTEGER; titles : COLLECTION[STRING]) is
      local 
	 string_array : STRING_ARRAY
      do
	 !!string_array.from_collection(titles)
	 gtkobject := ext_clist_new_with_titles(columns, 
						string_array.to_external)
      end

feature {ANY} -- Queries

   row_is_visible ( row : INTEGER ) : GTK_VISIBILITY is
      do
         !!Result.make(ext_clist_row_is_visible ( gtkobject, row ))
      end

   get_cell_type ( row, column : INTEGER ) : GTK_CELL_TYPE is 
      local cell_type : INTEGER
      do
	 cell_type := ext_clist_get_cell_type(gtkobject, row, column)
	 if (cell_type >= 0)  then 
	    !!Result.make(cell_type)
	 else
	    Result := Void
	 end
      end

   get_text ( row, column : INTEGER ) : STRING is
      local is_ok : INTEGER
      do
	 is_ok := ext_clist_get_text (gtkobject, row, column, $dummy)
	 if (is_ok > 0) then
	    !!Result.from_external(dummy)
	 else
	    Result := Void
	 end
      end

   get_pixtext_text ( row, column : INTEGER ) : STRING is
      local 
	 is_ok : INTEGER
      do
	 is_ok := ext_clist_get_pixtext ( gtkobject, row, column, $dummy, 
					  default_pointer, -- spacing
					  default_pointer, -- pixmap
					  default_pointer) -- mask
	 if (is_ok >0) then 
	    !!Result.from_external(dummy)
	 else
	    Result := Void
	 end
      end

   get_cell_style ( row, column : INTEGER ) : GTK_STYLE is
      do
	 !!Result.create(ext_clist_get_cell_style (gtkobject, row, column))
      end

   get_selectable ( row : INTEGER ) : BOOLEAN is
      do
	 Result := ext_clist_get_selectable ( gtkobject, row )
      end

feature {ANY} -- Stuff that's busted. 

   get_row_data ( row : INTEGER ) : ANY is
      do
	 Result := ext_clist_get_row_data( gtkobject, row ) 
      end

   find_row_from_data is
      do
      end

   get_selection_info is
      do
      end

feature {ANY} -- Commands

   set_cell_style ( row, column : INTEGER ; style_ : GTK_STYLE ) is
      do
	 ext_clist_set_cell_style ( gtkobject, row, column, 
				    style_.to_external )
      end

   set_row_style ( row : INTEGER ; style_ : GTK_STYLE ) is
      do
	 ext_clist_set_row_style ( gtkobject, row, style_.to_external )
      end

   set_selectable ( row : INTEGER ; selectable : BOOLEAN ) is
      do
	 ext_clist_set_selectable (gtkobject, row, selectable)
      end

   swap_rows ( row1, row2 : INTEGER ) is
      do
	 ext_clist_swap_rows ( gtkobject, row1, row2 )
      end

   row_move ( source_row, dest_row : INTEGER ) is
      do
	 ext_clist_row_move (gtkobject, source_row, dest_row) 
      end

   append ( text : COLLECTION[STRING] ) is
      local
	 string_array : STRING_ARRAY
      do
	 !!string_array.from_collection(text)
	 ext_clist_append(gtkobject, string_array.to_external)
      end

   insert ( row : INTEGER; text : COLLECTION[STRING] ) is
      local
	 string_array : STRING_ARRAY
      do
	 !!string_array.from_collection(text)
	 ext_clist_insert (gtkobject, row, string_array.to_external)
      end

   moveto ( row, column : INTEGER ; row_align, column_align : REAL ) is
      do
         ext_clist_moveto ( gtkobject, row, column, row_align, column_align )
      end

   remove (row : INTEGER) is 
      do
	 ext_clist_remove ( gtkobject, row ) 
      end

   select_row ( row, column : INTEGER ) is
      do
         ext_clist_select_row ( gtkobject, row, column )
      end

   set_background ( row : INTEGER ; color : GDK_COLOR ) is
      do
         ext_clist_set_background ( gtkobject, row, color.to_external )
      end

   set_foreground ( row : INTEGER ; color : GDK_COLOR ) is
      do
         ext_clist_set_foreground ( gtkobject, row, color.to_external )
      end

   set_pixmap ( row, column : INTEGER; pixmap : GDK_PIXMAP ) is
      do
	 ext_clist_set_pixmap ( gtkobject, row, column, 
				pixmap.to_external, pixmap.mask.to_external ) 
      end

   set_pixtext ( row, column : INTEGER; text : STRING ; spacing : INTEGER ; 
		 pixmap : GDK_PIXMAP ) is
      do
	 ext_clist_set_pixtext ( gtkobject, row, column, 
				 text.to_external, spacing,
				 pixmap.to_external, pixmap.mask.to_external )
      end

   set_row_data ( row : INTEGER ; data : ANY ) is
      do
	 ext_clist_set_row_data ( gtkobject, row, data ) 
      end

   set_shift ( row, column, vertical, horizontal : INTEGER ) is
      do
         ext_clist_set_shift ( gtkobject, row, column, vertical, horizontal )
      end

   set_text ( row, column : INTEGER ; text : STRING ) is
      do
         ext_clist_set_text ( gtkobject, row, column, text.to_external )
      end

   unselect_row ( row, column : INTEGER ) is
      do
         ext_clist_unselect_row ( gtkobject, row, column )
      end

feature {NONE} -- Private
   dummy : POINTER
   
   ext_clist_new ( columns : INTEGER ) : POINTER is
      external "C"
      alias "gtk_clist_new"
      end

   ext_clist_new_with_titles ( columns : INTEGER ; 
			       titles : POINTER ) : POINTER is
      external "C"
      alias "gtk_clist_new_with_titles"
      end

   ext_clist_append ( widget : POINTER ; text : POINTER ) is 
      external "C"
      alias "gtk_clist_append"
      end


   ext_clist_get_text ( clist : POINTER; row, column : 
			INTEGER ; text : POINTER ) : INTEGER is
      external "C"
      alias "gtk_clist_get_text"
      end

   ext_clist_get_row_data ( clist : POINTER; row : INTEGER ) : ANY is
      external "C"
      alias "gtk_clist_get_row_data"
      end

   ext_clist_insert ( clist : POINTER ; row : INTEGER ; text : POINTER ) is
      external "C"
      alias "gtk_clist_insert"
      end

   ext_clist_moveto ( clist : POINTER ; 
		      row, column : INTEGER ; 
		      row_align, column_align : REAL ) is
      external "C"
      alias "gtk_clist_moveto"
      end

   ext_clist_remove ( clist : POINTER ; row : INTEGER ) is
      external "C"
      alias "gtk_clist_remove"
      end

   ext_clist_row_is_visible ( clist : POINTER ; row : INTEGER ) : INTEGER is
      external "C"
      alias "gtk_clist_row_is_visible"
      end

   ext_clist_get_cell_type ( clist : POINTER ; 
			     row, column : INTEGER ) : INTEGER is
      external "C"
      alias "gtk_clist_get_cell_type"
      end

   ext_clist_select_row ( clist : POINTER ; row, column : INTEGER ) is
      external "C"
      alias "gtk_clist_select_row"
      end

   ext_clist_set_background ( clist : POINTER ; 
			      row : INTEGER ; color : POINTER ) is
      external "C"
      alias "gtk_clist_set_background"
      end

   ext_clist_set_foreground ( clist : POINTER ; 
			      row : INTEGER ; color : POINTER ) is
      external "C"
      alias "gtk_clist_set_foreground"
      end
 
   ext_clist_set_pixmap ( clist : POINTER ; row, column : INTEGER ; 
			  pixmap, mask : POINTER ) is
      external "C"
      alias "gtk_clist_set_pixmap"
      end

   -- This makes some questionable assumptions about stack layout, as
   -- spacing is a guint8. 
   ext_clist_set_pixtext ( clist : POINTER ; row, column : INTEGER ; 
			   text : POINTER ; spacing : INTEGER ; 
			   pixmap, mask : POINTER ) is
      external "C"
      alias "gtk_clist_set_pixtext"
      end
   
   ext_clist_set_row_data ( clist : POINTER ; row : INTEGER ; data : ANY ) is
      external "C"
      alias "gtk_clist_set_row_data"
      end


   ext_clist_set_shift ( clist : POINTER ; 
			 row, column, vertical, horizontal : INTEGER ) is
      external "C"
      alias "gtk_clist_set_shift"
      end

   ext_clist_set_text ( clist : POINTER ; 
			row, column : INTEGER ; text : POINTER ) is
      external "C"
      alias "gtk_clist_set_text"
      end

   ext_clist_unselect_row ( clist : POINTER ; row, column : INTEGER ) is
      external "C"
      alias "gtk_clist_unselect_row"
      end

   ext_clist_get_pixtext ( clist : POINTER ; row, column : INTEGER ; 
			   text, spacing, pixmap, mask : POINTER) : INTEGER is
      external "C"
      alias "gtk_clist_get_pixtext" 
      end

   ext_clist_set_cell_style ( clist : POINTER ; 
			      row, column : INTEGER ; style_ : POINTER ) is
      external "C"
      alias "gtk_clist_set_cell_style"
      end

   ext_clist_get_cell_style ( clist : POINTER ; 
			      row, column : INTEGER ) : POINTER is
      external "C"
      alias "gtk_clist_get_cell_style" 
      end
   
   ext_clist_set_row_style ( clist : POINTER ; 
			     row : INTEGER ; style_ : POINTER ) is
      external "C"
      alias "gtk_clist_set_row_style"
      end

   ext_clist_get_row_style ( clist : POINTER ; row : INTEGER ) : POINTER is
      external "C"
      alias "gtk_clist_get_row_style"
      end

   ext_clist_set_selectable ( clist : POINTER ; row : INTEGER ; 
			      selectable : BOOLEAN ) is
      external "C"
      alias "gtk_clist_set_selectable"
      end

   ext_clist_get_selectable ( clist : POINTER ; row : INTEGER ) : BOOLEAN is
      external "C"
      alias "gtk_clist_get_selectable"
      end

   ext_clist_swap_rows ( clist : POINTER ; row1, row2 : INTEGER ) is
      external "C"
      alias "gtk_clist_swap_rows"
      end
   
   ext_clist_row_move ( clist : POINTER ; source_row, dest_row : INTEGER ) is
      external "C"
      alias "gtk_clist_row_move"
      end

      
invariant
end
