indexing
   description: "gtk ctree"
   status: "You may be able to redistribute and/or modify this program.% 
          % See the README for details."
   copyright: "Copyright (C) 1999 Paul C. Janzen"
   author: "Paul C. Janzen"
   id: "$Id: gtk_ctree.e,v 1.12 1999/01/18 19:54:57 pcj Exp $"

class GTK_CTREE
inherit 
   GTK_CLIST_CTREE_TYPE

creation
   make_ctree,
   make_ctree_with_titles, 
   create

feature -- Creation

   make_ctree ( columns, tree_column : INTEGER ) is
      do
	 gtkobject := ext_ctree_new ( columns, tree_column ) 
      end

   make_ctree_with_titles ( columns, tree_column : INTEGER ; 
			    titles : COLLECTION[STRING] ) is
      local 
	 string_array : STRING_ARRAY
      do
	 !!string_array.from_collection(titles)
	 gtkobject := ext_ctree_new_with_titles ( columns, 
						  tree_column, 
						  string_array.to_external )
      end

feature {ANY} -- Queries

   is_viewable ( node : GTK_CTREE_NODE ) : BOOLEAN is 
      do
	 Result := ext_ctree_is_viewable ( gtkobject, node.to_external ) 
      end
   
   last ( node : GTK_CTREE_NODE ) : GTK_CTREE_NODE is
      do
	 !!Result.create(ext_ctree_last(gtkobject, node.to_external))
      end

   --   find_node_ptr ( ctree_row : GTK_CTREE_ROW ) : GTK_CTREE_NODE is
   --      do
   --	 !!Result.create(ext_ctree_find_node_ptr(gtkobject, 
   --						 ctree_row.to_external))
   --      end

   is_ancestor ( node, child : GTK_CTREE_NODE ) : BOOLEAN is 
      do
	 Result := ext_ctree_is_ancestor ( gtkobject, node.to_external, 
					   child.to_external ) 
      end

   is_hot_spot ( x, y : INTEGER ) : BOOLEAN is
      do
	 Result := ext_ctree_is_hot_spot ( gtkobject, x, y ) 
      end

   get_text ( node : GTK_CTREE_NODE ; column : INTEGER ) : STRING is
      local 
	 is_ok : INTEGER
      do
	 is_ok := ext_ctree_node_get_text ( gtkobject, node.to_external, 
					    column, $dummy )
	 if is_ok > 0 then
	    !!Result.from_external(dummy)
	 else
	    Result := Void
	 end
      end

   --   get_pixmap
   --   get_pixtext
   --   get_node_info

feature {ANY} -- Commands

   insert_node ( parent, sibling : GTK_CTREE_NODE; 
		 text : COLLECTION[STRING]; 
		 spacing : INTEGER; 
		 pixmap_closed, pixmap_open : GDK_PIXMAP; 
		 is_leaf, is_expanded : BOOLEAN ) : GTK_CTREE_NODE is
      local 
	 string_array : STRING_ARRAY
	 pptr, sptr : POINTER 
      do

	 if parent = Void then 
	    pptr := default_pointer
	 else 
	    pptr := parent.to_external
	 end

	 if sibling = Void then
	    sptr := default_pointer
	 else
	    sptr := sibling.to_external
	 end
	    
	 !!string_array.from_collection(text)
	 !!Result.create(ext_ctree_insert(gtkobject, 
					  pptr, sptr, 
					  string_array.to_external, 
					  spacing, 
					  pixmap_closed.to_external, 
					  pixmap_closed.mask.to_external,
					  pixmap_open.to_external,
					  pixmap_open.mask.to_external, 
					  is_leaf, is_expanded))
      end
   

   remove_node ( node : GTK_CTREE_NODE ) is
      do
	 ext_ctree_remove_node ( gtkobject, node.to_external ) 
      end

   move ( node, new_parent, new_sibling : GTK_CTREE_NODE ) is
      do
	 ext_ctree_move ( gtkobject, node.to_external, 
			  new_parent.to_external, new_sibling.to_external ) 
      end
   
   expand ( node : GTK_CTREE_NODE ) is
      do
	 ext_ctree_expand ( gtkobject, node.to_external ) 
      end

   expand_recursive ( node : GTK_CTREE_NODE ) is
      do
	 ext_ctree_expand_recursive ( gtkobject, node.to_external ) 
      end

   expand_to_depth ( node : GTK_CTREE_NODE ; depth : INTEGER ) is
      do
	 ext_ctree_expand_to_depth ( gtkobject, node.to_external, depth ) 
      end

   collapse ( node : GTK_CTREE_NODE ) is
      do
	 ext_ctree_collapse ( gtkobject, node.to_external ) 
      end

   collapse_recursive ( node : GTK_CTREE_NODE ) is
      do
	 ext_ctree_collapse_recursive ( gtkobject, node.to_external ) 
      end

   collapse_to_depth ( node : GTK_CTREE_NODE ; depth : INTEGER ) is
      do
	 ext_ctree_collapse_to_depth ( gtkobject, node.to_external, depth ) 
      end

   toggle_expansion ( node : GTK_CTREE_NODE ) is 
      do
	 ext_ctree_toggle_expansion ( gtkobject, node.to_external ) 
      end

   toggle_expansion_recursive ( node : GTK_CTREE_NODE ) is
      do
	 ext_ctree_toggle_expansion_recursive ( gtkobject, node.to_external ) 
      end

   selection ( node : GTK_CTREE_NODE ) is
      do
	 ext_ctree_select ( gtkobject, node.to_external ) 
      end
      
   selection_recursive ( node : GTK_CTREE_NODE ) is
      do
	 ext_ctree_select_recursive ( gtkobject, node.to_external ) 
      end

   unselect ( node : GTK_CTREE_NODE ) is
      do
	 ext_ctree_unselect ( gtkobject, node.to_external ) 
      end

   unselect_recursive ( node : GTK_CTREE_NODE ) is
      do
	 ext_ctree_unselect_recursive ( gtkobject, node.to_external ) 
      end

   real_select_recursive ( node : GTK_CTREE_NODE ; state : INTEGER ) is
      do
	 ext_ctree_real_select_recursive ( gtkobject, 
					   node.to_external, state ) 
      end

   set_text ( node : GTK_CTREE_NODE ; column : INTEGER ; text : STRING ) is
      do
	 ext_ctree_set_text ( gtkobject, node.to_external, 
			      column, text.to_external ) 
      end

   set_pixmap ( node : GTK_CTREE_NODE ; column : INTEGER ; 
		pixmap : GDK_PIXMAP ) is
      do
	 ext_ctree_set_pixmap ( gtkobject, node.to_external, column, 
				pixmap.to_external, pixmap.mask.to_external ) 
      end
		

   set_pixtext ( node : GTK_CTREE_NODE ; column : INTEGER ; text : STRING ; 
		 spacing : INTEGER ; pixmap : GDK_PIXMAP ) is
      do
	 ext_ctree_set_pixtext ( gtkobject, node.to_external, column, 
				 text.to_external, pixmap.to_external, 
				 pixmap.mask.to_external ) 
      end

   set_node ( node : GTK_CTREE_NODE ; text : STRING ; spacing : INTEGER ;
	      pixmap_closed, pixmap_open : GDK_PIXMAP ; 
	      is_leaf, is_expanded : BOOLEAN ) is
      do
	 ext_ctree_set_node_info ( gtkobject, 
				   node.to_external, 
				   text.to_external, 
				   spacing, 
				   pixmap_closed.to_external, 
				   pixmap_closed.mask.to_external, 
				   pixmap_open.to_external, 
				   pixmap_open.mask.to_external, 
				   is_leaf,
				   is_expanded ) 
      end
				   

   set_shift ( node : GTK_CTREE_NODE ; 
	       column, vertical, horizontal : INTEGER ) is
      do
	 ext_ctree_set_shift ( gtkobject, node.to_external, 
			       column, vertical, horizontal ) 
      end

   set_selectable ( node : GTK_CTREE_NODE ; selectable : BOOLEAN ) is
      do
	 ext_ctree_set_selectable ( gtkobject, node.to_external, selectable ) 
      end

   set_foreground ( node : GTK_CTREE_NODE ; color : GDK_COLOR ) is
      do
	 ext_ctree_set_foreground ( gtkobject, node.to_external, 
				    color.to_external ) 
      end

   set_background ( node : GTK_CTREE_NODE ; color : GDK_COLOR ) is
      do
	 ext_ctree_set_background ( gtkobject, node.to_external, 
				    color.to_external ) 
      end

   --   set_row_data
   --   set_row_data_full

   node_moveto ( node : GTK_CTREE_NODE ; column : INTEGER ; 
		 row_align, col_align : REAL ) is
      do
	 ext_ctree_node_moveto ( gtkobject, node.to_external, 
				 column, row_align, col_align ) 
      end

   set_indent ( indent : INTEGER ) is
      do
	 ext_ctree_set_indent ( gtkobject, indent ) 
      end
   
   set_line_style ( line_style : GTK_CTREE_LINE_STYLE ) is
      do
	 ext_ctree_set_line_style ( gtkobject, line_style.value ) 
      end

   --   set_drag_compare_func ( ) 


   sort_node ( node : GTK_CTREE_NODE ) is
      do
	 ext_ctree_sort_node ( gtkobject, node.to_external ) 
      end
   
   sort_recursive ( node : GTK_CTREE_NODE ) is
      do
	 ext_ctree_sort_recursive ( gtkobject, node.to_external ) 
      end

   set_show_stub ( show_stub_ : BOOLEAN ) is
      do
	 ext_ctree_set_show_stub ( gtkobject, show_stub_ ) 
      end

feature {NONE} -- Private

   ext_ctree_new ( columns, tree_column : INTEGER ) : POINTER is
      external "C"
      alias "gtk_ctree_new"
      end

   ext_ctree_new_with_titles ( columns, tree_column : INTEGER ; 
			       titles : POINTER ) : POINTER is
      external "C"
      alias "gtk_ctree_new_with_titles"
      end

   ext_ctree_is_viewable ( ctree, node : POINTER ) : BOOLEAN is
      external "C"
      alias "gtk_ctree_is_viewable"
      end

   ext_ctree_last ( ctree, node : POINTER ) : POINTER is
      external "C"
      alias "gtk_ctree_last"
      end

   ext_ctree_find_node_ptr ( ctree, node : POINTER ) : POINTER is
      external "C"
      alias "gtk_ctree_find_node_ptr"
      end

   ext_ctree_is_ancestor ( ctree, node, child : POINTER ) : BOOLEAN is
      external "C"
      alias "gtk_ctree_is_ancestor"
      end

   ext_ctree_is_hot_spot ( ctree : POINTER ; x, y : INTEGER ) : BOOLEAN is
      external "C"
      alias "gtk_ctree_is_hot_spot"
      end

   ext_ctree_node_get_text ( ctree, node : POINTER ; 
			     column : INTEGER ; text : POINTER) : INTEGER is
      external "C"
      alias "gtk_ctree_node_get_text" 
      end
	 
   ext_ctree_insert ( ctree, parent, sibling, text : POINTER ; 
		      spacing : INTEGER ; 
		      pixmap_closed, pixmap_closed_mask, 
		      pixmap_open, pixmap_open_mask : POINTER ; 
		      is_leaf, is_expanded : BOOLEAN ) : POINTER is
      external "C"
      alias "gtk_ctree_insert_node"
      end

   ext_ctree_remove_node ( ctree, node : POINTER ) is
      external "C"
      alias "gtk_ctree_remove_node"
      end

   ext_ctree_move ( ctree, node, new_parent, new_sibling : POINTER ) is
      external "C"
      alias "gtk_ctree_move"
      end

   ext_ctree_expand ( ctree, node : POINTER ) is
      external "C"
      alias "gtk_ctree_expand"
      end

   ext_ctree_expand_recursive ( ctree, node : POINTER ) is
      external "C"
      alias "gtk_ctree_expand_recursive"
      end

   ext_ctree_expand_to_depth ( ctree, node : POINTER ; depth : INTEGER ) is
      external "C"
      alias "gtk_ctree_expand_to_depth" 
      end

   ext_ctree_collapse ( ctree, node : POINTER ) is
      external "C"
      alias "gtk_ctree_collapse"
      end

   ext_ctree_collapse_recursive ( ctree, node : POINTER ) is
      external "C"
      alias "gtk_ctree_collapse_recursive"
      end

   ext_ctree_collapse_to_depth ( ctree, node : POINTER ; depth : INTEGER ) is
      external "C"
      alias "gtk_ctree_collapse_to_depth" 
      end

   ext_ctree_toggle_expansion ( ctree, node : POINTER ) is
      external "C"
      alias "gtk_ctree_toggle_expansion"
      end

   ext_ctree_toggle_expansion_recursive ( ctree, node : POINTER ) is
      external "C"
      alias "gtk_ctree_toggle_expansion_recursive"
      end

   ext_ctree_select ( ctree, node : POINTER ) is
      external "C"
      alias "gtk_ctree_select"
      end

   ext_ctree_select_recursive ( ctree, node : POINTER ) is
      external "C"
      alias "gtk_ctree_select_recursive"
      end

   ext_ctree_unselect ( ctree, node : POINTER ) is
      external "C"
      alias "gtk_ctree_unselect"
      end

   ext_ctree_unselect_recursive ( ctree, node : POINTER ) is
      external "C"
      alias "gtk_ctree_unselect_recursive"
      end

   ext_ctree_real_select_recursive ( ctree, node : POINTER ; 
				     state : INTEGER ) is
      external "C"
      alias "gtk_ctree_real_select_recursive"
      end   

   ext_ctree_set_text ( ctree, node : POINTER ; column : INTEGER ; 
			text : POINTER ) is
      external "C"
      alias "gtk_ctree_node_set_text"
      end

   ext_ctree_set_pixmap ( ctree, node : POINTER ; 
			  column : INTEGER ; pixmap, mask : POINTER ) is
      external "C"
      alias "gtk_ctree_node_set_pixmap"
      end

   ext_ctree_set_pixtext ( ctree, node : POINTER ; 
			   column : INTEGER ; 
			   text, pixmap, mask : POINTER ) is
      external "C"
      alias "gtk_ctree_node_set_pixtext"
      end
   
   ext_ctree_set_node_info ( ctree, node, text : POINTER ; spacing : INTEGER ; 
			     pixmap_closed, pixmap_closed_mask, 
			     pixmap_open, pixmap_open_mask : POINTER ; 
			     is_leaf , is_expanded : BOOLEAN ) is
      external "C"
      alias "gtk_ctree_set_node_info"
      end

   ext_ctree_set_shift ( ctree, node : POINTER ; 
			 column, vertical, horizontal : INTEGER ) is
      external "C"
      alias "gtk_ctree_node_set_shift"
      end

   ext_ctree_set_selectable ( ctree, node : POINTER ; 
			      selectable : BOOLEAN ) is 
      external "C"
      alias "gtk_ctree_node_set_selectable"
      end

   ext_ctree_set_foreground ( ctree, node, color : POINTER ) is
      external "C"
      alias "gtk_ctree_node_set_foreground"
      end

   ext_ctree_set_background ( ctree, node, color : POINTER ) is
      external "C"
      alias "gtk_ctree_node_set_background"
      end

   ext_ctree_node_moveto ( ctree, node : POINTER ; column : INTEGER ; 
			   row_align, col_align : REAL ) is
      external "C"
      alias "gtk_ctree_node_moveto"
      end
   
   ext_ctree_set_indent ( ctree : POINTER ; indent : INTEGER ) is
      external "C"
      alias "gtk_ctree_set_indent"
      end

   ext_ctree_set_line_style ( ctree : POINTER ; line_style : INTEGER ) is
      external "C"
      alias "gtk_ctree_set_line_style"
      end

   ext_ctree_sort_node ( ctree, node : POINTER ) is
      external "C"
      alias "gtk_ctree_sort_node"
      end

   ext_ctree_sort_recursive ( ctree, node : POINTER ) is
      external "C"
      alias "gtk_ctree_sort_recursive"
      end

   ext_ctree_set_show_stub ( ctree : POINTER ; show_stub_ : BOOLEAN ) is
      external "C"
      alias "gtk_ctree_set_show_stub"
      end

   dummy : POINTER 

invariant
end 
