indexing
   description: "gtk font selection"
   status: "You may be able to redistribute and/or modify this program.% 
          % See the README for details."
   copyright: "Copyright (C) 1999 Paul C. Janzen"
   author: "Paul C. Janzen"
   id: "$Id: gtk_font_selection.e,v 1.6 1999/01/18 19:54:57 pcj Exp $"

class GTK_FONT_SELECTION
inherit 
   GTK_NOTEBOOK
   redefine make end
creation
   make, create
feature -- Creation
   make is
      do
	 gtkobject := ext_font_selection_new 
      end -- make
feature {ANY} -- Queries

   get_font_name : STRING is
      local fn : POINTER
      do
	 fn := ext_font_selection_get_font_name(gtkobject)
	 if (fn.is_not_void) then 
	    !!Result.from_external(fn)
	 end
      end
   
   -- Remember to use font.ref/unref if you want to use the font, 
   -- in a style, for example. 
   get_font : GDK_FONT is
      do
	 !!Result.create(ext_font_selection_get_font(gtkobject))
      end

   set_font_name ( font : STRING ) : BOOLEAN is
      do
	 Result := ext_font_selection_set_font_name ( gtkobject, font.to_external )
      end

   set_filter (filter_type : GTK_FONT_FILTER_TYPE; 
	       font_type : GTK_FONT_TYPE; 
	       foundries, weights, slants, 
	       setwidths, spacings, charsets : COLLECTION[STRING]) is
      local
	 foundries_array, weights_array, slants_array, 
	 setwidths_array, spacings_array, charsets_array : STRING_ARRAY
      do
	 !!foundries_array.from_collection(foundries)
	 !!weights_array.from_collection(weights)
	 !!slants_array.from_collection(slants)
	 !!setwidths_array.from_collection(setwidths)
	 !!spacings_array.from_collection(spacings)
	 !!charsets_array.from_collection(charsets)
	 
	 ext_font_selection_set_filter(gtkobject, filter_type.value, 
				       font_type.value, 
				       foundries_array.to_external,
				       weights_array.to_external,
				       slants_array.to_external,
				       setwidths_array.to_external,
				       spacings_array.to_external,
				       charsets_array.to_external)
      end

   set_preview_text (text : STRING) is
      do
	 ext_font_selection_set_preview_text(gtkobject, text.to_external)
      end

   get_preview_text : STRING is
      local dummy : STRING
      do
	 !!dummy.from_external(ext_font_selection_get_preview_text(gtkobject))
	 !!Result.copy(dummy)
      end

feature {ANY} -- Commands
feature {NONE} -- Private

   ext_font_selection_new : POINTER is 
      external "C"
      alias "gtk_font_selection_new"
      end
   
   ext_font_selection_get_font_name ( fontsel : POINTER ) : POINTER is
      external "C"
      alias "gtk_font_selection_get_font_name"
      end

   ext_font_selection_get_font ( fontsel : POINTER ) : POINTER is
      external "C"
      alias "gtk_font_selection_get_font"
      end
   
   ext_font_selection_set_font_name ( fontsel, font : POINTER ) : BOOLEAN is
      external "C"
      alias "gtk_font_selection_set_font_name"
      end

   ext_font_selection_get_preview_text ( fontsel : POINTER ) : POINTER is
      external "C"
      alias "gtk_font_selection_get_preview_text"
      end

   ext_font_selection_set_preview_text ( fontsel, text : POINTER ) is
      external "C"
      alias "gtk_font_selection_set_preview_text"
      end

   ext_font_selection_set_filter(fontsel : POINTER; 
				 filter_type, font_type : INTEGER; 
				 foundries, weights, slants,
				 setwidths, spacings, charsets : POINTER) is
      external "C"
      alias "gtk_font_selection_set_filter"
      end

   -- FIXME 
   main_vbox, font_label, font_entry, font_clist, font_style_entry, 
   font_style_clist, size_entry, size_clist, pixels_button, points_button, 
   filter_button, scaled_bitmaps_button, preview_entry, message_label, 
   info_vbox, info_clist, requested_font_name, actual_font_name, 
   filter_vbox : GTK_WIDGET

   filter_clists: ARRAY[GTK_WIDGET]
invariant
end 

