indexing
   description: "gtk widget"
   status: "You may be able to redistribute and/or modify this program.% 
          % See the README for details."
   copyright: "Copyright (C) 1999 Paul C. Janzen"
   author: "Paul C. Janzen and Francois Pennaneach"
   id: "$Id: gtk_widget.e,v 1.20 1999/01/18 19:55:00 pcj Exp $"

class GTK_WIDGET

-- missing drag functions: drag_begin, drag_dest_handle_event, 
-- drag_dest_set_proxy, drag_get_source_widget, drag_set_icon_default,
-- drag_set_icon_pixmap, drag_set_icon_widget, 
-- drag_source_handle_event 

inherit
   GTK_OBJECT

creation
   make_default_object, 
   create

feature -- Creation
   make_default_object is
      do
	 gtkobject := ext_null_pointer
      end -- make
   
feature {ANY} -- Queries
   visible : BOOLEAN is
      do
	 Result := ext_widget_visible(gtkobject)
      end -- visible

   window : GDK_WINDOW is
      do
	 !!Result.create(ext_gdk_window_from_widget(gtkobject))
      end

   style : GTK_STYLE is
      do
	 !!Result.make_from_widget(Current)
      end

   allocation : GTK_ALLOCATION is 
      do
	 !!Result.create(ext_allocation(gtkobject))
      end

feature {ANY} -- Commands

   activate is
      do
	 ext_activate(gtkobject)
      end -- activate

   destroy is
      do
	 ext_widget_destroy(gtkobject)
      end -- destroy

   draw ( rectangle : GDK_OBJECT ) is
      do
	 ext_draw ( gtkobject, rectangle.to_external ) 
      end

   draw_default is
      do
	 ext_draw_default(gtkobject)
      end -- draw_default

   draw_focus is
      do
	 ext_draw_focus(gtkobject)
      end -- draw_focus
   
   get_name : STRING is
      do
	 !!Result.from_external(ext_get_name(gtkobject))
      end -- get_name

   -- grab_add from gtk_main
   grab_add is
      do
	 ext_grab_add(gtkobject)
      end -- grab_add

   grab_focus is
      do
	 ext_grab_focus(gtkobject)
      end -- grab_focus

   grab_default is
      do
	 ext_grab_default(gtkobject)
      end -- grab_default

   -- grab remove from gtk_main
   grab_remove is
      do
	 ext_grab_remove(gtkobject)
      end -- do

   hide is
	 -- hide this widget
      do
	 ext_hide(gtkobject)
      end -- hide

   hide_all is
      do
	 ext_widget_hide_all ( gtkobject ) 
      end

   add_accelerator( accel_signal : STRING ; 
		    accel_group : GTK_ACCEL_GROUP ; 
		    accel_key : INTEGER ; 
		    accel_mods : GDK_MODIFIER_TYPE ; 
		    accel_flags : GTK_ACCEL_FLAGS ) is
      require
	 accel_key > 0 
      do
	 ext_widget_add_accelerator ( gtkobject, 
				      accel_signal.to_external, 
				      accel_group.to_external, 
				      accel_key, 
				      accel_mods.value, 
				      accel_flags.value ) 
      end

   remove_accelerator ( accel_group : GTK_ACCEL_GROUP ; 
			accel_key : INTEGER ; 
			accel_mods : GDK_MODIFIER_TYPE ) is
      do
	 ext_widget_remove_accelerator ( gtkobject, 
					 accel_group.to_external,
					 accel_key, 
					 accel_mods.value ) 
      end

   remove_accelerators ( accel_signal : STRING ; 
			 visible_only : BOOLEAN ) is
      do
	 ext_widget_remove_accelerators ( gtkobject, accel_signal.to_external,
					  visible_only )
      end

   lock_accelerators is
      do
	 ext_widget_lock_accelerators ( gtkobject ) 
      end

   unlock_accelerators is
      do
	 ext_widget_unlock_accelerators ( gtkobject ) 
      end

   map is
      do
	 ext_map(gtkobject)
      end -- map

   popup(x, y : INTEGER) is
      do
	 ext_popup(gtkobject, x, y)
      end -- popup

   push_colormap ( cmap : GDK_COLORMAP ) is
      do
	 ext_push_colormap ( cmap.to_external ) 
      end

   pop_colormap is 
      external "C"
      alias "gtk_widget_pop_colormap"
      end
   
   push_style ( style_ : GTK_STYLE ) is
      do
	 ext_push_style ( style_.to_external ) 
      end

   pop_style is
      external "C"
      alias "gtk_widget_pop_style"
      end
   
   push_visual ( visual : GDK_VISUAL ) is
      do
	 ext_push_visual ( visual.to_external ) 
      end

   pop_visual is
      external "C"
      alias "gtk_widget_pop_visual"
      end

   rc_get_style : GTK_STYLE is
      do
	 !!Result.create(ext_rc_get_style(gtkobject))
      end -- rc_get_style

   set_flags(flags : GTK_OBJECT_FLAGS) is
      do
	 ext_widget_set_flags(gtkobject, flags.value)
      end -- set_flags

   set_name(name : STRING) is
      do
	 ext_set_name(gtkobject, name.to_external)
      end -- set_name

   set_parent(parent : GTK_WIDGET) is
      do
	 ext_set_parent(gtkobject, parent.to_external)
      end -- set_parent
   
   set_parent_window ( parent_window : GDK_WINDOW ) is
      do
	 ext_widget_set_parent_window ( gtkobject, parent_window.to_external ) 
      end

   set_sensitive(sensitive : BOOLEAN) is
      do
	 ext_set_sensitive(gtkobject, sensitive)
      end -- set_sensitive

   set_uposition(x, y : INTEGER) is
      do
	 ext_set_uposition(gtkobject, x, y)
      end -- set_uposition

   set_usize(width, height : INTEGER) is
      do
	 ext_set_usize(gtkobject, width, height)
      end -- set_usize

   size_allocate(allocation_ : GTK_ALLOCATION) is
      do
	 ext_size_allocate(gtkobject, allocation_.to_external)
      end -- size_allocate
   
   show is
      -- show this widget
      do
	 ext_show(gtkobject)
      end -- show

   show_all is
      -- Show this widget and all its children
      do
	 ext_show_all(gtkobject)
      end

   size_request(requisition : GTK_REQUISITION) is
      do
	 ext_size_request(gtkobject, requisition.to_external)
      end -- size_request

   set_events(events : GDK_EVENT_MASK) is
      do
	 ext_widget_set_events(gtkobject, events.value)
      end -- set_events

   realize is
      do
	 ext_realize(gtkobject)
      end -- realize

   reparent(new_parent : GTK_WIDGET) is
      do
	 ext_reparent(gtkobject, new_parent.to_external)
      end -- reparent

   unmap is
      do
	 ext_unmap(gtkobject)
      end -- unmap

   unparent is
      do
	 ext_unparent(gtkobject)
      end -- unparent

   unrealize is
      do
	 ext_unrealize(gtkobject)
      end -- unrealize

   -- uhh? 
   -- FIXME: GDK_EVENT_MOTION
   motion_notify_event( event : GDK_EVENT ) is
      do
	 ext_motion_notify_event (gtkobject, event.to_external)
      end

   -- Destination side

   drag_get_data (context : GDK_DRAG_CONTEXT; 
		  target : GDK_ATOM; 
		  time : INTEGER) is
      do
	 ext_drag_get_data(gtkobject, context.to_external, target.item, 
			   time)
      end

   drag_finish (context : GDK_DRAG_CONTEXT; 
		success, del : BOOLEAN; 
		time : INTEGER) is
      do
	 ext_drag_finish(context.to_external, success, del, time)
      end
   
   drag_highlight is
      do
	 ext_drag_highlight(gtkobject)
      end

   drag_unhighlight is
      do
	 ext_drag_unhighlight(gtkobject)
      end

   drag_dest_set ( flags : GTK_DEST_DEFAULTS ;
		   targets : COLLECTION[GTK_TARGET_ENTRY]; 
		   actions : GDK_DRAG_ACTION ) is
      local
	 targets_array : GTK_TARGET_ENTRY_ARRAY
      do
	 !!targets_array.from_collection(targets)
	 ext_drag_dest_set ( gtkobject, 
			     flags.value, 
			     targets_array.to_external,
			     targets.count, 
			     actions.value )
      end

   drag_dest_unset is
      do
	 ext_drag_dest_unset(gtkobject)
      end

   -- Source side

   drag_source_set ( start_button_mask : GDK_MODIFIER_TYPE; 
		     targets : COLLECTION[GTK_TARGET_ENTRY]; 
		     actions : GDK_DRAG_ACTION ) is
      local 
	 targets_array : GTK_TARGET_ENTRY_ARRAY
      do
	 !!targets_array.from_collection(targets)
	 ext_drag_source_set ( gtkobject, 
			       start_button_mask.value, 
			       targets_array.to_external,
			       targets.count, 
			       actions.value )
      end

   drag_source_unset is
      do
	 ext_drag_source_unset(gtkobject)
      end

   drag_source_set_icon(colormap : GDK_COLORMAP; 
			pixmap: GDK_PIXMAP) is
      do
	 ext_drag_source_set_icon(gtkobject, colormap.to_external, 
				  pixmap.to_external, pixmap.mask.to_external)
      end

   drag_set_default_icon ( colormap : GDK_COLORMAP ; 
			   pixmap : GDK_PIXMAP ; 
			   hot_x, hot_y : INTEGER ) is
      do
	 ext_drag_set_default_icon(colormap.to_external, 
				   pixmap.to_external, 
				   pixmap.mask.to_external,
				   hot_x, hot_y) 
      end

   -- Selection methods

   selection_owner_set ( selection : GDK_ATOM ; 
			 time : INTEGER ) is
      do
	 ext_selection_owner_set ( gtkobject, selection.to_external, 
				   time ) 
      end

   selection_add_target ( selection, target : GDK_ATOM ; 
			  info : INTEGER ) is
      do
	 ext_selection_add_target ( gtkobject, selection.to_external, 
				    target.to_external, info ) 
      end

   selection_convert ( selection, target : GDK_ATOM ; time : INTEGER ) is
      do
	 ext_selection_convert ( gtkobject, selection.to_external, 
				 target.to_external, time ) 
      end

feature {NONE} -- Private

   ext_widget_new : POINTER is
      external "C"
      alias "gtk_widget_new"
      end -- ext_widget_new
   
   ext_widget_destroy(c_object : POINTER) is
      external "C"
      alias "ext_widget_destroy"
      end -- ext_widget_destroy

   ext_activate(c_object : POINTER) is
      external "C"
      alias "gtk_widget_activate"
      end -- ext_activate

   ext_draw ( widget, rectangle : POINTER ) is
      external "C"
      alias "gtk_widget_draw"
      end

   ext_draw_default(c_object : POINTER) is
      external "C"
      alias "gtk_widget_draw_default"
      end -- ext_draw_default

   ext_draw_focus(c_object : POINTER) is
      external "C"
      alias "gtk_widget_draw_focus"
      end -- ext_draw_focus

   ext_get_name(c_object : POINTER) : POINTER is
      external "C"
      alias "gtk_widget_get_name"
      end -- ext_get_name

   ext_grab_add(c_object : POINTER) is
      external "C"
      alias "gtk_grab_add"
      end -- ext_grab_ad

   ext_grab_default(c_object : POINTER) is
      external "C"
      alias "gtk_widget_grab_default"
      end -- ext_grab_default

   ext_grab_focus(c_object : POINTER) is
      external "C"
      alias "gtk_widget_grab_focus"
      end -- ext_grab_focus

   ext_grab_remove(c_object : POINTER) is
      external "C"
      alias "gtk_grab_remove"
      end -- ext_grab_remove

   ext_hide(c_object : POINTER) is
      external "C"
      alias "gtk_widget_hide"
      end -- ext_hide

   ext_widget_hide_all (widget : POINTER) is
      external "C"
      alias "gtk_widget_hide_all"
      end

   ext_map(c_object : POINTER) is
      external "C"
      alias "gtk_widget_map"
      end -- ext_map

   ext_popup(c_object : POINTER; x, y: INTEGER) is
      external "C"
      alias "gtk_widget_popup"
      end -- ext_popup

   ext_push_colormap ( cmap : POINTER ) is
      external "C"
      alias "gtk_widget_push_colormap"
      end

   ext_push_style ( style_ : POINTER ) is
      external "C"
      alias "gtk_widget_push_style"
      end

   ext_push_visual ( visual : POINTER ) is
      external "C"
      alias "gtk_widget_push_visual"
      end

   ext_realize(c_object : POINTER) is
      external "C"
      alias "gtk_widget_realize"
      end -- ext_realize

   ext_rc_get_style(c_object : POINTER) : POINTER is
      external "C"
      alias "gtk_rc_get_style"
      end -- ext_rc_get_style

   ext_reparent(c_object, new_parent : POINTER) is
      external "C"
      alias "gtk_widget_reparent"
      end -- ext_reparent

   ext_widget_set_flags(c_object : POINTER; flags : INTEGER) is
      external "C"
      end -- ext_widget_set_flags

   ext_widget_set_events(c_object : POINTER; flags : INTEGER) is
      external "C"
      alias "gtk_widget_set_events"
      end -- ext_widget_set_events

   ext_set_name(c_object : POINTER; name : POINTER) is
      external "C"
      alias "gtk_widget_set_name"
      end -- ext_set_name

   ext_set_parent(c_object, parent : POINTER) is
      external "C"
      alias "gtk_widget_set_parent"
      end -- ext_set_parent

   ext_set_sensitive(c_object : POINTER; sensitive : BOOLEAN) is
      external "C"
      alias "gtk_widget_set_sensitive"
      end -- ext_set_sensitive

   ext_set_uposition(c_object : POINTER; x,y : INTEGER) is
      external "C"
      alias "gtk_widget_set_uposition"
      end -- ext_set_uposition

   ext_set_usize(c_object : POINTER; width, height : INTEGER) is
      external "C"
      alias "gtk_widget_set_usize"
      end -- ext_set_usize

   ext_show(c_object : POINTER) is
      external "C"
      alias "gtk_widget_show"
      end -- ext_show
   
   ext_show_all(c_object : POINTER) is
      external "C"
      alias "gtk_widget_show_all"
      end -- ext_show_all

   ext_size_allocate(c_object, allocation_ : POINTER) is
      external "C"
      alias "gtk_widget_size_allocate"
      end -- ext_size_allocate

   ext_allocation(c_object : POINTER ) : POINTER is
      external "C"
      end

   ext_size_request(c_object, requisition : POINTER) is
      external "C"
      alias "gtk_widget_size_request"
      end -- ext_size_request

   ext_unmap(c_object : POINTER) is
      external "C"
      alias "gtk_widget_unmap"
      end -- ext_unmap

   ext_unparent(c_object : POINTER) is
      external "C"
      alias "gtk_widget_unparent"
      end -- ext_unparent

   ext_unrealize(c_object : POINTER) is
      external "C"
      alias "gtk_widget_unrealize"
      end -- ext_unrealize

   ext_widget_visible(c_object : POINTER) : BOOLEAN is
      external "C"
      end -- ext_widget_visible

   ext_gdk_window_from_widget(c_object : POINTER) : POINTER is
      external "C"
      end
   
   ext_motion_notify_event ( widget, event : POINTER ) is
      external "C"
      end

   ext_drag_dest_set ( widget : POINTER ; 
		       flags : INTEGER ;
		       targets : POINTER ; 
		       n_targets : INTEGER ;
		       actions : INTEGER ) is
      external "C"
      end

   ext_drag_source_set ( widget : POINTER ;
			 start_button_mask : INTEGER ;
			 targets : POINTER ;
			 n_targets : INTEGER ;
			 actions : INTEGER ) is
      external "C"
      end

   ext_drag_get_data (widget, context : POINTER; target, time : INTEGER) is
      external "C"
      alias "gtk_drag_get_data"
      end
   
   ext_drag_finish (context : POINTER ; success, del : BOOLEAN; 
		    time : INTEGER ) is
      external "C"
      alias "gtk_drag_finish"
      end

   ext_drag_highlight (widget : POINTER) is
      external "C"
      alias "gtk_drag_highlight"
      end

   ext_drag_unhighlight(widget : POINTER) is
      external "C"
      alias "gtk_drag_unhighlight"
      end

   ext_drag_dest_unset(widget : POINTER) is
      external "C"
      alias "gtk_drag_dest_unset"
      end

   ext_drag_source_unset(widget : POINTER) is
      external "C"
      alias "gtk_drag_source_unset"
      end

   ext_drag_source_set_icon(widget, colormap, pixmap, mask : POINTER) is
      external "C"
      alias "gtk_drag_source_set_icon"
      end

   ext_drag_set_default_icon(colormap, pixmap, mask : POINTER; 
			     hot_x, hot_y : INTEGER) is
      external "C"
      alias "gtk_drag_set_default_icon"
      end

   ext_widget_add_accelerator ( widget, accel_signal, accel_group : POINTER ; 
				accel_key, accel_mods,
				accel_flags : INTEGER ) is
      external "C"
      alias "gtk_widget_add_accelerator"
      end

   ext_widget_remove_accelerator ( widget, accel_group : POINTER ; 
				   accel_key, accel_mods : INTEGER ) is
      external "C"
      alias "gtk_widget_remove_accelerator" 
      end

   ext_selection_owner_set ( widget : POINTER ; selection, time : INTEGER ) is
      external "C"
      alias "gtk_selection_owner_set"
      end

   ext_selection_add_target ( widget : POINTER ;
			      selection, target, info : INTEGER ) is
      external "C"
      alias "gtk_selection_add_target"
      end

   ext_selection_convert ( widget : POINTER ; 
			   selection, target, time : INTEGER ) is
      external "C"
      alias "gtk_selection_convert"
      end

   ext_widget_remove_accelerators ( widget, accel_signal : POINTER ; 
				    visible_only : BOOLEAN ) is
      external "C" 
      alias "gtk_widget_remove_accelerators"
      end

   ext_widget_lock_accelerators ( widget : POINTER ) is
      external "C"
      alias "gtk_widget_lock_accelerators"
      end

   ext_widget_unlock_accelerators ( widget : POINTER ) is
      external "C"
      alias "gtk_widget_unlock_accelerators"
      end

   ext_widget_set_parent_window ( widget, parent_window : POINTER ) is
      external "C"
      alias "gtk_widget_set_parent_window"
      end
   
invariant
end -- GTK_WIDGET

